"""Parses reflex config file to be used by application."""
import logging
import os

import pkg_resources

import yaml
from jinja2 import Environment, PackageLoader, select_autoescape
from reflex_cli.rule_discoverer import RuleDiscoverer
from reflex_cli.user_input import UserInput

TEMPLATE_FOLDER = os.path.abspath(os.path.join(os.path.dirname(__file__), "templates"))

LOGGER = logging.getLogger("reflex_cli")

REFLEX_PREFIX = "reflex-aws-"


class ReflexInitializer:
    """Creates assets required to build a reflex deployment."""

    def __init__(self, interactive, config_file):
        self.user_input = UserInput(interactive)
        self.configs = {}
        self.config_file = config_file
        self.rule_discoverer = RuleDiscoverer()
        self.template_env = Environment(
            loader=PackageLoader("reflex_cli", "templates"),
            autoescape=select_autoescape(["jinja2"]),
        )

    def query_possible_rules(self):
        """Iterates over templates and gets confirmation per rule."""
        discovered_rules = self.rule_discoverer.collect_rules()
        raw_rules = self.user_input.get_rule_input(discovered_rules)
        LOGGER.debug("Rules selected for config: %s", raw_rules)
        possible_rules = self.strip_rule_common_names(raw_rules)
        return possible_rules

    @staticmethod
    def strip_rule_common_names(rule_array):
        """Takes in array of rules and strips out common repo prefix."""
        stripped_array = []
        for rule in rule_array:
            rule_key = list(rule.keys())[0]
            stripped_key = rule_key.replace(REFLEX_PREFIX, "")
            new_rule = {stripped_key: rule[rule_key]}
            stripped_array.append(new_rule)
        return stripped_array

    @staticmethod
    def get_reflex_version():
        """Looks at package version of CLI to determine reflex version."""
        package_object = pkg_resources.require("reflex-cli")[0]
        LOGGER.debug("Reflex version found as: %s", package_object.version)
        return package_object.version

    def get_engine_version(self):
        """Pulls the engine version information from discoverer."""
        engine_dictionary = self.rule_discoverer.collect_engine()
        return engine_dictionary["reflex-engine"]["version"]

    def set_global_values(self):
        """Sets values for common configurations across guardrails."""
        global_dict = {}
        global_dict["default_email"] = self.user_input.collect_default_email()
        return global_dict

    def determine_config_values(self):  # pragma: no cover
        """Outlines keys of config file and gathers values."""
        self.configs["cli_version"] = self.get_reflex_version()
        self.configs["engine_version"] = self.get_engine_version()
        self.configs["globals"] = self.set_global_values()
        self.configs["providers"] = [{"aws": {"region": self.user_input.get_region()}}]
        self.configs["backend"] = self.user_input.get_backend_configuration()
        self.configs["rules"] = {"aws": self.query_possible_rules()}

    def render_template(self):  # pragma: no cover
        """Renders jinja2 template with yaml dumps."""
        version_dump = yaml.dump({"cli_version": ("%s" % self.configs["cli_version"])})
        engine_dump = yaml.dump(
            {"engine_version": ("%s" % self.configs["engine_version"])}
        )
        globals_dump = yaml.dump({"globals": self.configs["globals"]})
        providers_dump = yaml.dump({"providers": self.configs["providers"]})
        rules_dump = yaml.dump({"rules": self.configs["rules"]})
        backend_dump = yaml.dump({"backend": self.configs["backend"]})

        template = self.template_env.get_template("reflex.yaml.jinja2")
        rendered_template = template.render(
            global_configs=globals_dump,
            providers=providers_dump,
            backend=backend_dump,
            rules=rules_dump,
            version=version_dump,
            engine=engine_dump,
        )
        LOGGER.debug("Config template rendered as: %s", rendered_template)
        return rendered_template

    def write_config_file(self):  # pragma: no cover
        """Opens config file, dumps dict as yaml."""
        LOGGER.debug("Writing config file to: %s", self.config_file)
        if os.path.exists(self.config_file):
            write_file = self.user_input.ask_to_overwrite(self.config_file)
        else:
            write_file = True
        if write_file:
            with open(self.config_file, "w") as config_file:
                config_file.write(self.render_template())



"""
Upgrade command evaluates a current set of templates and
checks whether or not the deployed version could be updated.
"""
# pylint: disable=line-too-long
import logging
import os

import click

from reflex_cli.config_version_updater import ConfigVersionUpdater

LOGGER = logging.getLogger("reflex_cli")
CONFIG_DEFAULT = os.path.abspath(os.path.join(os.getcwd(), "reflex.yaml"))

SHORT_UPGRADE_HELP = "Evaluates existing reflex deployment for possible upgrades."
BOLD = "\033[1m"
ENDC = "\033[0m"


@click.command("upgrade", short_help=SHORT_UPGRADE_HELP)
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=True, dir_okay=False, resolve_path=True),
    default=CONFIG_DEFAULT,
    help="Configuration file for reflex.",
)
@click.option(
    "-a",
    "--all",
    "select_all",
    is_flag=True,
    help="Chooses to upgrade all possible rules.",
)
@click.option("-r", "--rule", help="Specify a rule that you would like to upgrade.")
def cli(rule, select_all, config):
    """
    Compares a local configuration file with external version information to allow users to upgrade rule versions automatically within their configuration.

    By default, configuration file name is set to be reflex.yaml.
    """
    LOGGER.info(
        "%s⏫ Determining if upgrade is needed for reflex deploy at in: %s%s",
        BOLD,
        config,
        ENDC,
    )
    updater = ConfigVersionUpdater(config, select_all)
    if rule:
        update_requested = updater.compare_current_rule_version(rule)
    else:
        update_requested = updater.compare_current_rule_versions()
        updater.upgrade_engine_version()
    if update_requested:
        updater.overwrite_reflex_config()



import os
import unittest

from reflex_cli import config_parser


class ConfigParserTestCase(unittest.TestCase):
    def setUp(self):
        self.test_config_file = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "resources/reflex.yaml")
        )
        self.test_config = config_parser.ConfigParser(self.test_config_file)

    def test_generate_config_returns_dictionary(self):
        self.test_config.parse_valid_config()
        self.assertTrue(isinstance(self.test_config.raw_configuration, dict))

    def test_validate_config(self):
        empty_config = {}
        self.assertFalse(self.test_config.validate_config(empty_config))

        config_key_no_value = {"version": None}
        self.assertFalse(self.test_config.validate_config(config_key_no_value))

        self.test_config.parse_valid_config()
        self.assertTrue(
            self.test_config.validate_config(self.test_config.raw_configuration)
        )



import unittest
from unittest.mock import  patch, mock_open

from reflex_cli.create_template_utils import ensure_output_directory_exists, write_template_file


class TemplateUtilsTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    @patch('pathlib.Path.mkdir')
    def test_create_rule_list(self, mock_mkdir):
        ensure_output_directory_exists("/test")
        mock_mkdir.assert_called_with(parents=True, exist_ok=True)

    def test_write_template_file(self):
        with patch("builtins.open", mock_open()) as open_mock:
            write_template_file("test.txt", "test")
            open_mock.assert_called_with('test.txt', 'w+')
            open_mock.return_value.write.assert_called_with("test")




"""
Create is used to generate a skeleton for a new Reflex rule
"""
import logging
import os

import click

from reflex_cli.repo_foundation_skeleton import RepoFoundationSkeleton
from reflex_cli.terraform_skeleton import TerraformSkeleton

LOGGER = logging.getLogger(__name__)
BOLD = "\033[1m"
ENDC = "\033[0m"


@click.command("create", short_help="Create a new reflex rule.")
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False, dir_okay=True, file_okay=False, resolve_path=True),
    help="Output directory for this reflex rule",
)
@click.option(
    "-r",
    "--rule-name",
    type=str,
    help="The kebab-case name for your rule. Example: my-encryption-rule",
    prompt="The kebab-case name for your rule.",
)
@click.option(
    "-c",
    "--class-name",
    type=str,
    help="The PascalCase name for your rule. Example: MyEncryptionRule",
    prompt="The PascalCase name for your rule.",
)
@click.option(
    "-m",
    "--mode",
    type=str,
    help="The mode for your rule. Options: DETECT | REMEDIATE",
    prompt="The mode for your rule. [DETECT | REMEDIATE]",
)
@click.option(
    "-g",
    "--github-org",
    type=str,
    help="The organization or user that will store this repository.",
    prompt="The organization or user that will store this repository.",
)
def cli(output, rule_name, class_name, mode, github_org):
    """
    Creates a skeleton rule directory to enable the faster creation of custom reflex rules.

    For further information on creating rules for reflex, check out https://docs.cloudmitigator.com.
    """
    if output:
        output_directory = os.path.abspath(os.path.join(os.getcwd(), output))
    else:
        output_directory = os.path.abspath(os.path.join(os.getcwd(), rule_name))

    skeleton_configuration = {
        "rule_name": rule_name,
        "class_name": class_name,
        "mode": mode,
        "github_org": github_org,
    }

    foundation_generator = RepoFoundationSkeleton(
        output_directory=output_directory, configuration=skeleton_configuration
    )
    foundation_generator.create_templates()

    terraform_generator = TerraformSkeleton(
        output_directory=output_directory, configuration=skeleton_configuration
    )
    terraform_generator.create_templates()

    LOGGER.info("%s🎨 Generating custom rule template files...%s", BOLD, ENDC)



import unittest
from unittest.mock import patch

import click

from reflex_cli import reflex_cli


class ReflexCliTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    def test_is_command_file(self):
        """Test our defaults for the environment are sane."""
        test_cli = reflex_cli.ReflexCli()
        self.assertTrue(test_cli.is_command_file("command_test.py"))
        self.assertFalse(test_cli.is_command_file("comd_test.py"))

    @patch("os.listdir")
    def test_list_commands(self, mock):
        mock.return_value = ["test"]
        test_empty_cli = reflex_cli.ReflexCli()
        self.assertEqual([], test_empty_cli.list_commands({}))

        mock.return_value = ["command_test.py"]
        self.assertEqual(["test"], test_empty_cli.list_commands({}))

    def test_get_command(self):
        test_cli_build_command = reflex_cli.ReflexCli()
        build_cli = test_cli_build_command.get_command({}, "build")

        self.assertTrue(isinstance(build_cli, click.core.Command))



"""Holder of template generation logic"""
import copy
import logging
import os
from pathlib import Path

from jinja2 import Environment, PackageLoader, select_autoescape

LOGGER = logging.getLogger("reflex_cli")
DEFAULT_GITHUB_ORG = "cloudmitigator"
BOLD = "\033[1m"
ENDC = "\033[0m"


class RegionTemplateGenerator:
    """Generate a set of templates from a given config."""

    def __init__(self, configuration, output_directory, region):
        self.configuration = configuration
        self.output_directory = output_directory
        self.region = region
        self.template_env = Environment(
            loader=PackageLoader("reflex_cli", "templates"),
            autoescape=select_autoescape(["tf"]),
        )

    def create_templates(self):  # pragma: no cover
        """Generates templates for every rule in configuration."""
        self.create_reflex_kms_template()
        self.create_provider_templates()
        self.create_region_rule_templates()
        self.create_backend_template()

    def create_reflex_kms_template(self):  # pragma: no cover
        """Generates template for central sns topic infrastructure."""
        template = self.template_env.get_template("reflex-kms-key.tf")
        rendered_template = template.render(
            engine_version=self.configuration["engine_version"]
        )
        self.build_output_file(["reflex-kms-key"], rendered_template)

    def create_provider_templates(self):  # pragma: no cover
        """Creates a simpler provider output file in terraform."""
        for provider in self.configuration["providers"]:
            template = self.template_env.get_template("provider.tf")
            rendered_template = template.render(
                provider_name=list(provider)[0], region_name=self.region
            )
            self.build_output_file(["providers"], rendered_template)

    def create_region_rule_templates(self):
        """Creates tf file for each rule"""
        for rule in self.configuration["rules"]["aws"]:
            rendered_template = self.generate_template("aws-rule-region.tf", rule)
            if rendered_template:
                self.build_output_file(rule, rendered_template)

    def create_backend_template(self):
        """Creates tf file for the Terraform backend"""
        if self.configuration["backend"]:
            backend_type = list(self.configuration["backend"])[0]
            if backend_type != "s3":
                return
            template = self.template_env.get_template("backend.tf")
            config_copy = copy.deepcopy(self.configuration)
            for config in config_copy["backend"][backend_type]:
                for key, _ in config.items():
                    if key == "key":
                        config[key] += f"-{self.region}"
            rendered_template = template.render(
                backend_type=backend_type,
                backend_config_array=config_copy["backend"][backend_type],
            )
            self.build_output_file(["backend"], rendered_template)

    def generate_template(self, template_name, rule):  # pragma: no cover
        """Creates tf output for every file in our template."""
        template = self.template_env.get_template(template_name)
        rule_name = list(rule)[0]

        github_org = rule[rule_name].get("github_org", DEFAULT_GITHUB_ORG)
        if github_org == DEFAULT_GITHUB_ORG:
            repo_name = f"reflex-aws-{rule_name}"
        else:
            repo_name = rule_name
        name_list = [item.capitalize() for item in rule_name.split("-")]
        camel_name = "".join(name_list)
        rendered_template = template.render(
            module_name=f"{rule_name}-{self.region}",
            template_name=repo_name,
            version=rule[rule_name]["version"],
            engine_version=self.configuration["engine_version"],
            github_org=github_org,
            central_region=self.configuration["providers"][0]["aws"]["region"],
            queue_name=camel_name,
            configuration=rule[rule_name].get("configuration"),
        )
        LOGGER.debug(rendered_template)
        return rendered_template

    @staticmethod
    def build_template_names_from_rule(rule):
        """Returns rule name with tf appended."""
        return list(rule)[0] + ".tf"

    def build_output_file(self, rule, rendered_template):
        """Build output file name to write rendering to file"""
        template_name = self.build_template_names_from_rule(rule)
        output_file = os.path.join(self.output_directory, template_name)
        self.write_template_file(output_file, rendered_template)

    def write_template_file(self, output_file, rendered_template):  # pragma: no cover
        """Writes output of rendering to file"""
        self._ensure_output_directory_exists()
        LOGGER.info("📃 Writing terraform file to: %s%s%s", BOLD, output_file, ENDC)
        with open(output_file, "w+") as file_handler:
            file_handler.write(rendered_template)

    def _ensure_output_directory_exists(self):  # pragma: no cover
        """Ensure that the path to the output directory exists."""
        Path(self.output_directory).mkdir(parents=True, exist_ok=True)



import os
import unittest

from reflex_cli import cli_environment


class CliEnvironmentTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    def test_environment_defaults(self):
        """Test our defaults for the environment are sane."""
        fresh_environment = cli_environment.CliEnvironment()
        self.assertFalse(fresh_environment.verbose)
        self.assertEqual(fresh_environment.home, os.getcwd())



"""Properties for our Rule object in the CLI."""

DEFAULT_VARIABLES = [
    "sns_topic_arn",
    "reflex_kms_key_id",
    "cloudwatch_event_rule_id",
    "cloudwatch_event_rule_arn",
]


class Rule:
    """Base class that represents a rule object to the CLI."""

    def __init__(self, rule_name, rule_dict):
        self.name = rule_name
        self.raw_rule = rule_dict
        self.version = rule_dict.get("version")
        self.description = rule_dict.get("description")
        self.variables = rule_dict.get("variables")

    @property
    def configurables(self):
        """Displays configurable elements of a Rule."""
        configurables = []
        for variable in self.variables:
            if variable not in DEFAULT_VARIABLES:
                configurables.append(variable)
        return configurables

    @property
    def numeric_version(self):
        """Returns a version dictionary for major minor patch."""
        raw_version = self.version.replace("v", "")
        split_version = raw_version.split(".")
        return {
            "major": split_version[0],
            "minor": split_version[1],
            "patch": split_version[2],
        }



"""
Region command takes in configuration file outputs region
forwarder infrastructure template
"""
# pylint: disable=line-too-long
import logging
import os

import click

from reflex_cli.config_parser import ConfigParser
from reflex_cli.region_template_generator import RegionTemplateGenerator

CONFIG_DEFAULT = os.path.abspath(os.path.join(os.getcwd(), "reflex.yaml"))
OUTPUT_DEFAULT = os.path.abspath(os.path.join(os.getcwd(), "reflex_region"))
LOGGER = logging.getLogger("reflex_cli")
BOLD = "\033[1m"
ENDC = "\033[0m"


@click.command(
    "region", short_help="Builds out tf files for region forwarder architecture.",
)
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False, dir_okay=True, file_okay=False, resolve_path=True),
    default=OUTPUT_DEFAULT,
    help="Output directory for reflex",
)
@click.option(
    "-r",
    "--region",
    help="Region to deploy central forwarding infrastructure.",
    required=True,
)
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=True, dir_okay=False, resolve_path=True),
    default=CONFIG_DEFAULT,
    help="Configuration file for reflex",
)
def cli(output, region, config):
    """
    Builds terraform output for multi-region support of reflex in AWS.

    Region will look at your configuration, generate the relevant templates
    and providers for deploying region forwarders.
    """
    LOGGER.debug("Config file set to: %s", config)
    configuration = ConfigParser(config)
    configuration.parse_valid_config()
    if output == OUTPUT_DEFAULT:
        output = OUTPUT_DEFAULT + "_" + region.replace("-", "_")
    generator = RegionTemplateGenerator(configuration.raw_configuration, output, region)
    LOGGER.info(
        "✍  %s Writing regional terraform files for %s ... %s", BOLD, region, ENDC,
    )
    generator.create_templates()



import os
import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.region_template_generator import RegionTemplateGenerator

EXAMPLE_CONFIG = {
    "version": 0.1,
    "engine_version": "0.2.55",
    "globals": {"default_email": "john@example.com"},
    "providers": [{"aws": {"region": "us-east-1"}}],
    "backend": None,
    "rules": [
        "aws-detect-root-user-activity",
        {"aws-detect-deactivate-mfa": {"email": "john.smith@example.com"}},
        "aws-enforce-no-public-ami",
        "aws-enforce-s3-encryption",
        {"aws-detect-pants-on-fire": {"url": "github.com/example/pants"}},
    ],
}

OUTPUT_DIRECTORY = os.getcwd()


class TemplateGeneratorTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    def setUp(self):
        self.generator = RegionTemplateGenerator(
            EXAMPLE_CONFIG, OUTPUT_DIRECTORY, "us-east-2"
        )

    def test_generate_template(self):
        get_template_mock = MagicMock()
        render_mock = MagicMock()
        render_mock.render.return_value = "Example"
        get_template_mock.get_template.return_value = render_mock
        self.generator.template_env = get_template_mock
        self.assertEqual(
            self.generator.generate_template("test", {"test": {"version": "0.0.0.1"}}),
            "Example",
        )

    @patch(
        "reflex_cli.region_template_generator.RegionTemplateGenerator.create_reflex_kms_template"
    )
    @patch(
        "reflex_cli.region_template_generator.RegionTemplateGenerator.create_provider_templates"
    )
    @patch(
        "reflex_cli.region_template_generator.RegionTemplateGenerator.create_region_rule_templates"
    )
    @patch(
        "reflex_cli.region_template_generator.RegionTemplateGenerator.create_backend_template"
    )
    def test_create_templates(
        self,
        create_backend_template_mock,
        create_region_rule_templates_mock,
        create_provider_templates_mock,
        create_reflex_kms_template_mock,
    ):
        self.generator.create_templates()
        create_backend_template_mock.assert_called_once()
        create_region_rule_templates_mock.assert_called_once()
        create_provider_templates_mock.assert_called_once()
        create_reflex_kms_template_mock.assert_called_once()

    def test_build_template_names_from_rule(self):
        rule = {"my_rule_name": "data"}
        response = self.generator.build_template_names_from_rule(rule)
        assert response == "my_rule_name.tf"

    def test_create_backend_template_returns_none_no_backend(self):
        assert self.generator.create_backend_template() is None

    @patch("reflex_cli.template_generator.TemplateGenerator.write_template_file")
    def test_write_template_file(self, write_template_file_mock):
        write_template_file_mock.return_value = True



""" Creates templates for new Reflex rules """
import logging
import os

from jinja2 import Environment, PackageLoader, select_autoescape
from reflex_cli.create_template_utils import (
    ensure_output_directory_exists,
    write_template_file,
)
from reflex_cli.rule_discoverer import RuleDiscoverer

LOGGER = logging.getLogger(__name__)


class TerraformSkeleton:
    """Generate a set of templates from a given config."""

    def __init__(self, output_directory, configuration):
        self.output_directory = output_directory
        self.rule_name = configuration.get("rule_name")
        self.class_name = configuration.get("class_name")
        self.engine_version = self.get_engine_version()
        self.mode = configuration.get("mode")
        self.template_env = Environment(
            loader=PackageLoader("reflex_cli", "templates/rule_templates"),
            autoescape=select_autoescape(["tf"]),
        )

    def create_templates(self):  # pragma: no cover
        """ Generates templates for rule. """
        ensure_output_directory_exists(self.output_directory)
        self.create_cwe_terraform_template()
        self.create_cwe_output_template()
        self.create_sqs_lambda_terraform_template()
        self.create_variables_terraform_template()

    def create_template(
        self, template_file, template_options, output_path
    ):  # pragma: no cover
        """Helper method to create file from rendered jinja."""
        template = self.template_env.get_template(template_file)
        rendered_template = template.render(template_options)
        output_file = os.path.join(self.output_directory, output_path)
        write_template_file(output_file, rendered_template)

    def create_cwe_terraform_template(self):  # pragma: no cover
        """ Generates a .tf module for our rule """
        self.create_template(
            template_file="cwe.tf",
            template_options={
                "rule_class_name": self.class_name,
                "engine_version": self.engine_version,
            },
            output_path="terraform/cwe/cwe.tf",
        )

    def create_cwe_output_template(self):  # pragma: no cover
        """ Generates a .tf module for our rule """
        self.create_template(
            template_file="output.tf",
            template_options={},
            output_path="terraform/cwe/output.tf",
        )

    def create_sqs_lambda_terraform_template(self):  # pragma: no cover
        """ Generates a .tf module for our rule """
        self.create_template(
            template_file="sqs_lambda.tf",
            template_options={
                "rule_name": self.rule_name,
                "rule_class_name": self.class_name,
                "mode": self.mode,
                "engine_version": self.engine_version,
            },
            output_path="terraform/sqs_lambda/sqs_lambda.tf",
        )

    def create_variables_terraform_template(self):  # pragma: no cover
        """Creates tf output for every file in our template."""
        self.create_template(
            template_file="variables.tf",
            template_options={"mode": self.mode},
            output_path="terraform/sqs_lambda/variables.tf",
        )

    @staticmethod
    def get_engine_version():
        """ Pulls current engine version from manifest."""
        measure_manifest = RuleDiscoverer()
        engine_dictionary = measure_manifest.collect_engine()
        return engine_dictionary["reflex-engine"]["version"]



""" Creates templates for new Reflex rules """
import logging
import os

from jinja2 import Environment, PackageLoader, select_autoescape
from reflex_cli.create_template_utils import (
    ensure_output_directory_exists,
    write_template_file,
)

LOGGER = logging.getLogger(__name__)
DEFAULT_GITHUB_ORG = "cloudmitigator"


class RepoFoundationSkeleton:
    """Generate a set of templates from a given config."""

    def __init__(
        self, output_directory, configuration
    ):  # pylint: disable=too-many-arguments
        self.output_directory = output_directory
        self.github_org_name = configuration.get("github_org")
        self.rule_name = configuration.get("rule_name")
        self.class_name = configuration.get("class_name")
        self.mode = configuration.get("mode")
        self.template_env = Environment(
            loader=PackageLoader("reflex_cli", "templates/rule_templates"),
            autoescape=select_autoescape(["tf"]),
        )

    def create_templates(self):  # pragma: no cover
        """ Generates templates for rule. """
        ensure_output_directory_exists(self.output_directory)
        self.create_workflow_template()
        self.create_source_template()
        self.create_requirements_template()
        self.create_gitignore_template()
        self.create_license_template()
        self.create_readme_template()

    def create_template(
        self, template_file, template_options, output_path
    ):  # pragma: no cover
        """Helper method to create file from rendered jinja."""
        template = self.template_env.get_template(template_file)
        rendered_template = template.render(template_options)
        output_file = os.path.join(self.output_directory, output_path)
        write_template_file(output_file, rendered_template)

    def create_workflow_template(self):  # pragma: no cover
        """ Generates template for GitHub release file """
        self.create_template(
            template_file=".github/workflows/release.yaml.jinja2",
            template_options={},
            output_path=".github/workflows/release.yaml",
        )

    def create_source_template(self):  # pragma: no cover
        """ Generates template for rule source code """
        self.create_template(
            template_file="source/rule.py.jinja2",
            template_options={"rule_class_name": self.class_name, "mode": self.mode,},
            output_path=f"source/{self.rule_name.replace('-', '_')}.py",
        )

    def create_requirements_template(self):  # pragma: no cover
        """ Generates template for requirements.txt """
        require_file = "source/requirements.txt"
        self.create_template(
            template_file=require_file, template_options={}, output_path=require_file,
        )

    def create_gitignore_template(self):  # pragma: no cover
        """ Generates template for .gitignore """
        self.create_template(
            template_file=".gitignore", template_options={}, output_path=".gitignore",
        )

    def create_license_template(self):  # pragma: no cover
        """ Generates template for LICENSE """
        self.create_template(
            template_file="LICENSE", template_options={}, output_path="LICENSE"
        )

    def create_readme_template(self):  # pragma: no cover
        """ Generates template for README.md """
        self.create_template(
            template_file="README.md",
            template_options={
                "github_org_name": self.github_org_name,
                "rule_name": self.rule_name,
            },
            output_path="README.md",
        )



import os
import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.user_input import UserInput


class UserInputTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    def setUp(self):
        self.user_input = UserInput(True)

    @patch("reflex_cli.user_input.UserInput.get_input")
    def test_collect_default_email(self, input_mock):
        """Test our logic for rules is correct"""
        input_mock.return_value = "example@example.com"
        select_all_input = UserInput(False)
        self.assertEqual(
            "placeholder@example.com", select_all_input.collect_default_email()
        )

        no_select_all_input = UserInput(True)
        self.assertEqual(
            "example@example.com", no_select_all_input.collect_default_email()
        )

    @patch("reflex_cli.user_input.UserInput.get_input")
    def test_verify_rule(self, input_mock):
        """Test our logic for rules is correct"""
        input_mock.return_value = "y"
        test = MagicMock()
        test.name = "example"
        test.version = "example"
        self.assertTrue(self.user_input.verify_rule(test))

    @patch("reflex_cli.user_input.UserInput.verify_rule")
    def test_get_rule_input(self, input_mock):
        """Test our logic for rules is correct"""
        input_mock.return_value = True
        test = MagicMock()
        test.name = "example"
        test.version = None

        self.assertEqual([], self.user_input.get_rule_input([test]))

        test2 = MagicMock()
        test2.name = "first"
        test2.version = "example"
        test2.configurables = None

        self.assertEqual(
            [{"first": {"version": "example"}}],
            self.user_input.get_rule_input([test2]),
        )

        test3 = MagicMock()
        test3.name = "first"
        test3.version = "example"
        test3.configurables = ["mode"]

        self.assertEqual(
            [{"first": {"version": "example", "configuration": {"mode": ""}}}],
            self.user_input.get_rule_input([test3]),
        )

        test4 = MagicMock()
        test4.name = "first"
        test4.version = "example"
        test4.configurables = [{"mode": "detect"}]

        self.assertEqual(
            [
                {
                    "first": {
                        "version": "example",
                        "configuration": {"mode": "detect"},
                    }
                }
            ],
            self.user_input.get_rule_input([test4]),
        )

    @patch("reflex_cli.user_input.UserInput.get_input")
    def test_get_backend_key_values(self, input_mock):
        """Test our logic for rules is correct"""
        input_mock.return_value = "N"
        key_values = self.user_input.collect_backend_key_values()
        self.assertEqual(key_values, [{"N": "N"}])

    @patch("reflex_cli.user_input.UserInput.get_input")
    @patch("reflex_cli.user_input.UserInput.collect_backend_key_values")
    def test_get_backend_config(self, key_values, input_mock):
        """Test our logic for rules is correct"""
        input_mock.return_value = "y"
        key_values.return_value = [{"bucket": "s3-bucket"}]
        backend_config = self.user_input.get_backend_configuration()
        self.assertEqual(backend_config, {"y": [{"bucket": "s3-bucket"}]})

    @patch("reflex_cli.user_input.UserInput.get_input")
    @patch("os.environ.get")
    def test_get_region(self, environment_variables, input_mock):
        """Test our logic for rules is correct"""
        input_mock.return_value = "us-east-1"
        environment_variables.return_value = None
        all_input = UserInput(False)
        self.assertIsNone(all_input.get_region())
        self.assertEqual("us-east-1", self.user_input.get_region())

    @patch("reflex_cli.user_input.UserInput.get_input")
    @patch("reflex_cli.user_input.UserInput.ask_to_overwrite")
    def test_verify_upgrade_interested(self, overwrite_mock, input_mock):
        """Test our logic for rules is correct"""
        overwrite_mock.return_value = True
        all_input = UserInput(False)
        self.assertTrue(all_input.verify_upgrade_interest())



"""Class to create decorator for command classes."""
# pylint: disable=too-few-public-methods
import os


class CliEnvironment:
    """Helper class for click specific commands."""

    def __init__(self):
        self.verbose = False
        self.home = os.getcwd()



"""Class that takes in user input for various CLI pieces"""
import logging
import os

from PyInquirer import prompt

LOGGER = logging.getLogger("reflex_cli")
HEADER = "\033[95m"
OKBLUE = "\033[94m"
OKGREEN = "\033[92m"
WARNING = "\033[93m"
FAIL = "\033[91m"
ENDC = "\033[0m"
BOLD = "\033[1m"
UNDERLINE = "\033[4m"


class UserInput:
    """Class for handling user input."""

    def __init__(self, interactive):
        self.interactive = interactive

    @staticmethod
    def get_input(message):  # pragma: no cover
        """Helper method to return specific input call."""
        return input(message)

    def collect_default_email(self):
        """Collects default email address from user."""
        if not self.interactive:
            return "placeholder@example.com"
        return self.get_input("Default email:")

    def verify_rule(self, rule):
        """Gets user input about rule inclusion."""
        verify_string = f"Add {rule.name} at version {rule.version}?"
        rule_input = self.get_input(verify_string + " (Yy/Nn):")
        return rule_input.lower() == "y"

    def get_rule_input(self, discovered_rules):
        """Iterates over all rules and verifies inputs."""
        possible_rules = []
        for rule in discovered_rules:
            if rule.version is None:
                continue
            if not self.interactive or self.verify_rule(rule):
                LOGGER.info(
                    "✅ Adding %s%s%s%s at version %s%s%s",
                    OKGREEN,
                    BOLD,
                    rule.name,
                    ENDC,
                    OKBLUE,
                    rule.version,
                    ENDC,
                )
                if rule.configurables:
                    configurable_dict = {}
                    for config in rule.configurables:
                        LOGGER.debug("Adding element %s to config array.", config)
                        if isinstance(config, str):
                            configurable_dict[config] = ""
                        elif isinstance(config, dict):
                            config_key = list(config)[0]
                            configurable_dict[config_key] = config[config_key]
                    possible_rules.append(
                        {
                            rule.name: {
                                "version": rule.version,
                                "configuration": configurable_dict,
                            }
                        }
                    )
                else:
                    possible_rules.append({rule.name: {"version": rule.version}})
        return possible_rules

    def collect_backend_key_values(self):  # pragma: no cover
        """Continually collects backend config key value pairs from user."""
        more_config = True
        key_value_array = []
        while more_config:
            config_key = self.get_input("Backend configuration key: ")
            config_value = self.get_input("Backend configuration value: ")
            key_value_array.append({config_key: config_value})
            continue_config = self.get_input("Add more configurations? (Yy/Nn):")
            more_config = continue_config.lower() == "y"
        return key_value_array

    def get_backend_configuration(self):  # pragma: no cover
        """Collects backend configuration information."""
        if self.interactive:
            backend_verify = self.get_input("Configure backend? (Yy/Nn): ")
            if backend_verify.lower() != "y":
                return ""
            LOGGER.info(
                "Collecting backend configuration. To see key value configuration,"
                " check out https://www.terraform.io/docs/backends/index.html."
            )
            backend_type = self.get_input("Backend type?: ")
            backend_config = {backend_type: self.collect_backend_key_values()}
            return backend_config
        return None

    def get_region(self):
        """Either sets region via environment variable or input."""
        region = os.environ.get("AWS_REGION")
        if not region and self.interactive:
            region = self.get_input("AWS Region:")
        return region

    @staticmethod
    def ask_to_overwrite(file_path):
        """Asks user whether to allow overwrite file. """
        configuration_overwrite = [
            {
                "type": "confirm",
                "message": f"Configuration file found at {file_path}, overwrite?",
                "name": "overwrite_configuration",
                "default": False,
            }
        ]
        overwrite = prompt(configuration_overwrite)
        return overwrite["overwrite_configuration"]

    def verify_upgrade_interest(self):
        """Prompts user whether or not they want to upgrade rule."""
        if self.interactive:
            upgrade_interest = [
                {
                    "type": "confirm",
                    "message": "Upgrade?",
                    "name": "upgrade_interest",
                    "default": False,
                }
            ]
            verify = prompt(upgrade_interest)
            return verify["upgrade_interest"]
        return True



"""
Show command is used for discovery of detective rules that you may
want to include in your reflex configuration.
"""
import logging

import click

from reflex_cli.rule_discoverer import RuleDiscoverer

LOGGER = logging.getLogger("reflex_cli")


@click.command("show", short_help="Shows possible reflex rules.")
def cli():
    """
    Displays the currently supported common rules provided by reflex.

    Requires access to reflex's internal system manifest at manifest.cloudmitigator.com.
    """
    discoverer = RuleDiscoverer()
    LOGGER.info("Collecting list of available rules...")
    discoverer.collect_rules()
    discoverer.display_discovered_rules()



"""Parses reflex config file to be used by application."""
import logging
import sys

# pylint: disable=wrong-import-order
import yaml
from reflex_cli.rule import Rule
from yaml.constructor import SafeConstructor

LOGGER = logging.getLogger("reflex_cli")
REQUIRED_KEYS = [
    "cli_version",
    "globals",
    "rules",
    "providers",
    "backend",
    "engine_version",
]


def add_bool(self, node):
    """Allows bools to be set as strings."""
    return self.construct_scalar(node)


SafeConstructor.add_constructor("tag:yaml.org,2002:bool", add_bool)


class ConfigParser:
    """Parses and validates reflex yaml config file."""

    def __init__(self, config_file):
        self.config_file = config_file
        self.raw_coniguration = {}
        self.rule_list = []

    def parse_valid_config(self):
        """Entrypoint to generate and validate config"""
        self.raw_configuration = (  # pylint: disable=attribute-defined-outside-init
            self.parse_yaml_config()
        )
        self.rule_list = self.create_rule_list()
        valid_config = self.validate_config(self.raw_configuration)
        if not valid_config:
            LOGGER.info(
                "Invalid configuration file format found at %s", self.config_file,
            )
            sys.exit(55)

    def create_rule_list(self):
        """Creates rule objects in a list for further ingestion."""
        rule_object_array = []
        try:
            for rule in self.raw_configuration["rules"]["aws"]:
                rule_name = list(rule)[0]
                new_rule = Rule(rule_name, rule[rule_name])
                rule_object_array.append(new_rule)
            return rule_object_array
        except KeyError:
            LOGGER.info(
                "Parsing config failed: Incorrect configuration"
                " rule structure in %s",
                self.config_file,
            )
            sys.exit(55)

    def parse_yaml_config(self):
        """Opens config file, parses yaml."""
        with open(self.config_file, "r") as config_file:
            configuration = yaml.safe_load(config_file)
            LOGGER.debug("Configuration dictionary: %s", configuration)
        return configuration

    @staticmethod
    def validate_config(config):
        """Validates presence of keys and format of values"""
        valid = True
        config_list = list(config)

        for key in REQUIRED_KEYS:
            if key not in config_list:
                LOGGER.info("Key %s was not found in reflex.yaml.", key)
                valid = False
            elif not config[key] and key != "backend":
                LOGGER.info("Key %s has no value in reflex.yaml", key)
                valid = False
        return valid



import os
import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.config_version_updater import ConfigVersionUpdater


class ConfigVersionUpdaterTestCase(unittest.TestCase):
    def setUp(self):
        self.test_config_file = os.path.abspath(
            os.path.join(os.path.dirname(__file__), "resources/reflex.yaml")
        )
        self.test_config_updater = ConfigVersionUpdater(
            self.test_config_file, False
        )

    def test_find_rule_value(self):
        real_value = self.test_config_updater._find_rule_value(
            "aws-detect-root-user-activity", "version"
        )
        self.assertEqual(real_value, "v0.0.1")
        no_value = self.test_config_updater._find_rule_value(
            "aws-detect-root-user-activity", "empty"
        )
        self.assertIsNone(no_value)

    def test_set_rule_value(self):
        self.test_config_updater._set_rule_value(
            "aws-detect-root-user-activity", "version", "invalid"
        )
        new_value = self.test_config_updater._find_rule_value(
            "aws-detect-root-user-activity", "version"
        )
        self.assertEqual(new_value, "invalid")
        self.test_config_updater._set_rule_value(
            "aws-detect-root-user-activity", "version", "v0.0.1"
        )

    @patch("reflex_cli.config_version_updater.RuleDiscoverer.collect_rules")
    def test_gather_latest_remote_versions(self, rule_mock):
        rule_list_mock = MagicMock()
        rule_list_mock.name = "aws-detect-root-user-activity"
        rule_list_mock.version = "v0.0.1"
        collect_rule_mock = MagicMock()
        collect_rule_mock.name = "reflex-aws-aws-detect-root-user-activity"
        collect_rule_mock.version = "v0.0.1"
        rule_mock.return_value = [collect_rule_mock]
        self.test_config_updater.current_config.rule_list = [rule_list_mock]
        latest_versions = (
            self.test_config_updater.gather_latest_remote_versions()
        )
        self.assertTrue(isinstance(latest_versions, dict))
        self.assertEqual(
            latest_versions["aws-detect-root-user-activity"], "v0.0.1"
        )

    @patch(
        "reflex_cli.config_version_updater.ConfigVersionUpdater.gather_latest_remote_versions"
    )
    @patch(
        "reflex_cli.config_version_updater.UserInput.verify_upgrade_interest"
    )
    def test_compare_current_rule_versions(
        self, user_input_mock, remote_version_mock
    ):
        rule_list_mock = MagicMock()
        rule_list_mock.name = "aws-detect-root-user-activity"
        rule_list_mock.version = "v0.0.1"
        self.test_config_updater.current_config.rule_list = [rule_list_mock]
        user_input_mock.return_value = True
        remote_version_mock.return_value = {
            "aws-detect-root-user-activity": "v0.0.2"
        }
        self.assertEqual(
            "v0.0.1",
            self.test_config_updater._find_rule_value(
                "aws-detect-root-user-activity", "version"
            ),
        )
        self.test_config_updater.compare_current_rule_versions()
        self.assertEqual(
            "v0.0.2",
            self.test_config_updater._find_rule_value(
                "aws-detect-root-user-activity", "version"
            ),
        )



import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.rule_discoverer import RuleDiscoverer


class RuleDiscovererTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    def setUp(self):
        self.discoverer = RuleDiscoverer()

    def test_create_rule_list(self):
        test_rules = [{"test-rule": {"description": "example"}}]
        rule_objects = self.discoverer.create_rule_list(test_rules)
        self.assertEqual(rule_objects[0].name, "test-rule")



import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.rule import Rule


class RuleTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    def test_configurables(self):
        example_rule = Rule("test", {"variables": ["one", "sns_topic_arn"]})
        self.assertEqual(example_rule.configurables, ["one"])



"""Holder of template generation logic"""
import base64
import logging
import os

import github
import hcl

LOGGER = logging.getLogger("reflex_cli")


class ReflexGithub:
    """Discovers rules by querying github remotes."""

    def __init__(self):
        self.github_organizations = ["cloudmitigator"]

        token = os.environ.get("REFLEX_GITHUB_TOKEN")
        self.github_client = github.Github(token)

    def get_repos(self):  # pragma: no cover
        """Iterates over github org and collects repos that match rules."""
        repositories = []

        for organization in self.github_organizations:  # pragma: no cover
            LOGGER.debug("Collecting repos for %s", organization)
            org_api = self.github_client.get_organization(organization)
            repositories += org_api.get_repos()

        return repositories

    def get_remote_version(self, remote):
        """Calls github API for remote to get latest release."""
        try:  # pragma: no cover
            repo = self.github_client.get_repo(remote)
        except github.GithubException:  # pragma: no cover
            LOGGER.debug("No remote resource found at github.com/%s", remote)
            return None
        try:  # pragma: no cover
            latest_release = repo.get_latest_release()
        except github.GithubException:  # pragma: no cover
            LOGGER.debug("No releases found for %s", remote)
            return None
        return latest_release.tag_name

    def get_rule_mode(self, remote):  # pragma: no cover
        """
        Calls GitHub API and determines if the specified rule has different modes

        If no modes are available, returns None. If modes are available, returns "remediate"
        """
        try:  # pragma: no cover
            repo = self.github_client.get_repo(remote)
        except github.GithubException:  # pragma: no cover
            LOGGER.warning("No remote resource found at github.com/%s", remote)
            return None

        try:
            contents = repo.get_contents("variables.tf")
            content_string = base64.b64decode(contents.content).decode("ascii")
            content_dict = hcl.loads(content_string)
            if "mode" in content_dict["variables"]:
                LOGGER.debug("Found 'mode' variable in %s", remote)
                return "remediate"
            return None
        except Exception as exception:  # pylint: disable=broad-except
            LOGGER.warning(
                "Something went wrong when trying to determine mode for %s", remote,
            )
            LOGGER.exception(exception)
            return None

    @staticmethod
    def get_repo_format(remote):
        """Takes in a repo URL and returns its repo format."""
        org_repo_string = None
        github_string = remote.find("github.com/")
        if github_string > 0:
            org_repo_string = "/".join(
                remote[github_string + 11 :].split("/")  # noqa: E203
            )
        LOGGER.debug("Found repo string to be: %s", org_repo_string)
        return org_repo_string



"""Entrypoint for our reflex CLI application."""
# pylint: disable=invalid-name,no-value-for-parameter
import logging
import os
import sys

import click

from reflex_cli.cli_environment import CliEnvironment
from reflex_cli.reflex_cli import ReflexCli

FORMAT = "%(message)s"
logging.basicConfig(stream=sys.stdout, format=FORMAT, level=logging.INFO)
LOGGER = logging.getLogger("reflex_cli")

CONFIG_DEFAULT = os.path.abspath(os.path.join(os.getcwd(), "reflex.yaml"))
CONTEXT_SETTINGS = {"auto_envvar_prefix": "reflex"}

pass_environment = click.make_pass_decorator(CliEnvironment, ensure=True)


@click.command(cls=ReflexCli, context_settings=CONTEXT_SETTINGS)
@click.version_option()
@click.option(
    "--home",
    type=click.Path(exists=True, file_okay=False, resolve_path=True),
    help="Changes the folder to operate on.",
)
@click.option("-v", "--verbose", is_flag=True, help="Enables verbose mode.")
@pass_environment
def cli(context, verbose, home):
    """
    CLI interface for building infrastructure to deploy the reflex cloud security framework.

    To learn more about reflex, check out https://docs.cloudmitigator.com.

    Get started by building a configuration file with `reflex init`.
    """
    context.verbose = verbose
    if context.verbose:
        LOGGER.setLevel(logging.DEBUG)
    if home is not None:
        context.home = home
    LOGGER.debug("home set to: %s", context.home)


if __name__ == "__main__":
    cli()



"""Parses reflex config file to be used by application."""
import logging

from reflex_cli.config_parser import ConfigParser
from reflex_cli.reflex_github import ReflexGithub
from reflex_cli.reflex_initializer import ReflexInitializer
from reflex_cli.rule_discoverer import RuleDiscoverer
from reflex_cli.user_input import UserInput

LOGGER = logging.getLogger("reflex_cli")


class ConfigVersionUpdater:
    """Checks remote sources for newer rule releases"""

    def __init__(self, config_file, select_all):

        self.user_input = UserInput(not select_all)
        self.config_file = config_file
        self.current_config = ConfigParser(self.config_file)
        self.current_config.parse_valid_config()

    def gather_latest_remote_versions(self):
        """Reaches out to urls to get tag information per rule."""
        manifest_rules = RuleDiscoverer().collect_rules()
        remote_versions = {}
        for rule in self.current_config.rule_list:
            remote_url = self._find_rule_value(rule.name, "url")
            if not remote_url:
                cleaned_rule_name = f"reflex-aws-{rule.name}"
                for manifest_rule in manifest_rules:
                    if manifest_rule.name == cleaned_rule_name:
                        remote_versions[rule.name] = manifest_rule.version
            else:
                LOGGER.debug("Rule: %s has remote: %s", rule, remote_url)
                remote_versions[rule.name] = ReflexGithub().get_remote_version(
                    ReflexGithub.get_repo_format(remote_url)
                )
            LOGGER.debug("rule has remote version: %s", remote_versions[rule.name])
        return remote_versions

    def upgrade_engine_version(self):
        """Checks for the latest engine version"""
        engine_data = RuleDiscoverer().collect_engine()
        engine_version = engine_data["reflex-engine"]["version"]
        self.current_config.raw_configuration["engine_version"] = engine_version

    def _find_rule_value(self, rule_name, key):
        for rule in self.current_config.raw_configuration["rules"]["aws"]:
            if rule.get(rule_name):
                return rule.get(rule_name).get(key)
        return None

    def _set_rule_value(self, rule_name, key, value):
        """Overwrites existing key with value."""
        for rule in self.current_config.raw_configuration["rules"]["aws"]:
            if rule.get(rule_name):
                rule[rule_name][key] = value

    def compare_current_rule_versions(self):
        """Iterates over all rules and compares rules with remote versions."""
        LOGGER.debug("Comparing current rule versions.")
        update_requested = False
        remote_versions = self.gather_latest_remote_versions()
        for rule in self.current_config.rule_list:
            current_version = self._find_rule_value(rule.name, "version")
            remote_version = remote_versions[rule.name]
            if not remote_version:
                LOGGER.debug("No release information for %s. Skipping!", rule.name)
                continue
            if current_version != remote_version:
                LOGGER.info(
                    "%s (current version: %s) has new release: %s.",
                    rule.name,
                    current_version,
                    remote_version,
                )
                if self.user_input.verify_upgrade_interest():
                    update_requested = True
                    self._set_rule_value(rule.name, "version", remote_version)
        return update_requested

    def compare_current_rule_version(self, rule_name):
        """ Check single rule version and compare it to remote version."""
        update_requested = False
        rule_found = False
        remote_versions = self.gather_latest_remote_versions()
        for rule in self.current_config.rule_list:
            if rule.name == rule_name:
                rule_found = True
                current_version = self._find_rule_value(rule.name, "version")
                remote_version = remote_versions[rule.name]
                if not remote_version:
                    LOGGER.info("No release information for %s. Skipping!", rule.name)
                    return update_requested
                if current_version != remote_version:
                    LOGGER.info(
                        "%s (current version: %s) has new release: %s.",
                        rule.name,
                        current_version,
                        remote_version,
                    )
                    if self.user_input.verify_upgrade_interest():
                        update_requested = True
                        self._set_rule_value(rule.name, "version", remote_version)
                    return update_requested
                LOGGER.info("%s rule does not have a new release.", rule.name)
                return update_requested
        if not rule_found:
            LOGGER.info(
                "Rule with name %s does not exist, please check the spelling.",
                rule_name,
            )
        return update_requested

    def overwrite_reflex_config(self):
        """If any upgrades possible, overwrite current reflex config."""
        initializer = ReflexInitializer(False, self.config_file)
        initializer.config_file = self.config_file
        initializer.configs = self.current_config.raw_configuration
        initializer.write_config_file()



import os
import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.reflex_initializer import ReflexInitializer


class ReflexInitializerTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    def setUp(self):
        self.initializer = ReflexInitializer(False, os.getcwd())

    @patch("reflex_cli.reflex_initializer.RuleDiscoverer")
    @patch("reflex_cli.reflex_initializer.UserInput.get_input")
    def test_query_possible_rules(self, input_mock, discoverer_mock):
        """Test our logic for rules is correct"""
        input_mock.return_value = "y"
        rule_mock = MagicMock()
        rule_mock.name = "test"
        rule_mock.version = "test"
        discoverer_mock = rule_mock
        discoverer_mock.discovered_rules = [rule_mock]
        test_object = ReflexInitializer(os.getcwd(), False)
        single_valid_template = test_object.query_possible_rules()
        self.assertEqual(single_valid_template, [])

        discoverer_mock.return_value = ["test"]
        no_valid_template = test_object.query_possible_rules()
        self.assertTrue(no_valid_template == [])

    @patch("reflex_cli.reflex_initializer.pkg_resources")
    def test_get_reflex_version(self, pkg_mock):
        test_object = ReflexInitializer(False, os.getcwd())
        version_mock = MagicMock()
        version_mock.version = "4.3.21"
        pkg_mock.require.return_value = [version_mock]
        self.assertTrue(test_object.get_reflex_version() == "4.3.21")

    def test_strip_rule_common_names(self):
        example_rule_array = [
            {"reflex-aws-test": {"example": "example1"}},
            {"reflex-aws-example": "value"},
            {"non-reflex": "value"},
        ]
        desired_rule_array = [
            {"test": {"example": "example1"}},
            {"example": "value"},
            {"non-reflex": "value"},
        ]
        processed_array = self.initializer.strip_rule_common_names(
            example_rule_array
        )
        self.assertNotEqual(example_rule_array, processed_array)
        self.assertEqual(processed_array, desired_rule_array)

    @patch("reflex_cli.reflex_initializer.UserInput.collect_default_email")
    def test_set_global_values(self, input_mock):
        input_mock.return_value = "example@example.com"
        empty_initializer = ReflexInitializer(False, os.getcwd())
        self.assertTrue("globals" not in empty_initializer.configs.keys())
        self.assertEqual(
            "example@example.com",
            empty_initializer.set_global_values()["default_email"],
        )



"""Holder of template generation logic"""
import logging
import sys

import requests
import yaml
from reflex_cli.rule import Rule

LOGGER = logging.getLogger("reflex_cli")

RULE_MANIFEST_ENDPOINT = "http://manifest.cloudmitigator.com"


class RuleDiscoverer:
    """Discovers rules by querying github remotes."""

    def __init__(self):
        self.discovered_rules = []
        self.manifest_content = self.pull_manifest_content()
        self.yaml_content = yaml.load(self.manifest_content, Loader=yaml.SafeLoader)

    def collect_rules(self):  # pragma: no cover
        """Collects a list of repos that match rules."""
        self.discovered_rules = self.create_rule_list(self.yaml_content["rules"])
        return self.discovered_rules

    def collect_engine(self):
        """Collects engine configuration from manifest content"""
        return self.yaml_content["engine"]

    @staticmethod
    def create_rule_list(rule_array):
        """Creates rule objects in a list for further ingestion."""
        rule_object_array = []
        for rule in rule_array:
            rule_name = list(rule)[0]
            new_rule = Rule(rule_name, rule[rule_name])
            rule_object_array.append(new_rule)
        return rule_object_array

    @staticmethod
    def pull_manifest_content():  # pragma: no cover
        """Reaches out to manifest endpoint to gather rule information."""
        try:
            raw_rules_response = requests.get(RULE_MANIFEST_ENDPOINT)
        except requests.ConnectionError:
            LOGGER.error(
                "A connection error occurred. Check your internet connection and try again."
            )
            sys.exit(1)
        return raw_rules_response.content

    def display_discovered_rules(self):  # pragma: no cover
        """Method outputs rule information in a usable fashion"""
        LOGGER.info("Rules discovered in CloudMitigator Github organization.")
        LOGGER.info("-------------------------------------------")
        for rule in self.discovered_rules:
            if rule.version:
                LOGGER.info(
                    "\033[4m%s\033[0m: \n\t\033[4mDescription\033[0m:"
                    " %s\n\t\033[4mLatest release\033[0m: %s\n",
                    rule.name,
                    rule.description,
                    rule.version,
                )



import os
import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.repo_foundation_skeleton import RepoFoundationSkeleton


class RepoFoundationSkeletonTestCase(unittest.TestCase):
    def setUp(self):
        configuration_object = {
            "github_org": "test",
            "rule_name": "unit-test-rule",
            "class_name": "UnitTestRule",
            "mode": "detect",
        }
        self.repo_foundation_skeleton = RepoFoundationSkeleton(
            output_directory="test_dir", configuration=configuration_object
        )

    @patch("reflex_cli.repo_foundation_skeleton.write_template_file")
    def test_create_template(self, write_template_file):
        self.repo_foundation_skeleton.create_template(
            "source/rule.py.jinja2",
            {
                "rule_class_name": self.repo_foundation_skeleton.class_name,
                "mode": self.repo_foundation_skeleton.mode,
            },
            f"source/{self.repo_foundation_skeleton.rule_name.replace('-','_')}.py",
        )
        write_template_file.assert_called_once()



import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.reflex_github import ReflexGithub


class ReflexGithubTestCase(unittest.TestCase):
    """Test class for the environment context of our CLI tool."""

    def test_get_repo_format(self):
        repo_string = ReflexGithub.get_repo_format(
            "http://github.com/test/repository"
        )
        self.assertEqual(repo_string, "test/repository")

        bad_string = ReflexGithub.get_repo_format(
            "http://githb.com/test/repository"
        )
        self.assertIsNone(bad_string)

    # def setUp(self):
    #     self.client = ReflexGithub()


#    @patch("reflex_cli.reflex_github.github.Github")
#    def test_get_repos_calls_organization_get_repos(self, mock_github):
#        """ Tests that get_repos calls the GitHub api get_repos function """
#        get_repos = MagicMock()
#        get_organizations = MagicMock()
#        get_organizations.get_repos.return_value = get_repos
#        mock_github.return_value = get_organizations
#        client = ReflexGithub()
#        client.get_repos()
#        mock_github.get_repos.assert_called_once()
#
# def test_get_remote_version_returns_none_no(self):
#     self.assertTrue(self.discoverer.is_rule_repository("reflex-aws-test"))
#     self.assertFalse(self.discoverer.is_rule_repository("aws-test"))



"""Command that will create a bootstrapped directory for reflex."""
import logging
import os

import click

from reflex_cli.reflex_initializer import ReflexInitializer

LOGGER = logging.getLogger("reflex_cli")
CONFIG_DEFAULT = os.path.abspath(os.path.join(os.getcwd(), "reflex.yaml"))
BOLD = "\033[1m"
ENDC = "\033[0m"


@click.command("init", short_help="Initializes a directory with a configuration file.")
@click.option(
    "-i",
    "--interactive",
    "interactive",
    is_flag=True,
    default=False,
    help="Prompt user for decision on which individual rules to add to configuration file.",
)
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=False, dir_okay=False, resolve_path=True),
    default=CONFIG_DEFAULT,
    help="Configuration file for reflex",
)
def cli(interactive, config):
    """
    Generates a reflex configuration yaml file to be later used to build reflex infrastructure.

    Default configuration file name is reflex.yaml
    """
    LOGGER.debug("Initializing reflex directory in: %s", config)
    LOGGER.info("%s📝 Generating reflex.yaml config file in: %s%s", BOLD, config, ENDC)
    initializer = ReflexInitializer(interactive, config)
    initializer.determine_config_values()
    initializer.write_config_file()



"""Build command takes in configuration file outputs infrastructure template"""
# pylint: disable=line-too-long
import logging
import os

import click

from PyInquirer import prompt
from reflex_cli.config_parser import ConfigParser
from reflex_cli.region_template_generator import RegionTemplateGenerator
from reflex_cli.template_generator import TemplateGenerator

REGION_DEFAULT = os.path.abspath(os.path.join(os.getcwd(), "reflex_region"))
CONFIG_DEFAULT = os.path.abspath(os.path.join(os.getcwd(), "reflex.yaml"))
OUTPUT_DEFAULT = os.path.abspath(os.path.join(os.getcwd(), "reflex_out"))
LOGGER = logging.getLogger("reflex_cli")
PLACEHOLDER_EMAIL = "placeholder@example.com"
BOLD = "\033[1m"
ENDC = "\033[0m"


@click.command("build", short_help="Builds out tf files from config file.")
@click.option(
    "-o",
    "--output",
    type=click.Path(exists=False, dir_okay=True, file_okay=False, resolve_path=True),
    default=OUTPUT_DEFAULT,
    help="Output directory for reflex",
)
@click.option(
    "-c",
    "--config",
    type=click.Path(exists=True, dir_okay=False, resolve_path=True),
    default=CONFIG_DEFAULT,
    help="Configuration file for reflex",
)
def cli(output, config):
    """
    Builds terraform output files for reflex infrastructure based upon input configuration file.

    By default, build looks for configuration in a local reflex.yaml and outputs built infrastructure to a reflex_out directory.
    """
    LOGGER.debug("Config file set to: %s", config)
    LOGGER.debug("Output directory set to: %s", output)
    configuration = ConfigParser(config)
    configuration.parse_valid_config()
    if configuration.raw_configuration["globals"]["default_email"] == PLACEHOLDER_EMAIL:
        questions = [
            {
                "type": "confirm",
                "message": "Notification email is left as placeholder. Edit before build?",
                "name": "edit_email",
                "default": True,
            }
        ]
        answers = prompt(questions)
        if answers["edit_email"]:
            email_form = [
                {
                    "type": "input",
                    "name": "default_email",
                    "message": "Default email address for reflex notifications:",
                }
            ]
            email = prompt(email_form)["default_email"]
            configuration.raw_configuration["globals"]["default_email"] = email
    generator = TemplateGenerator(configuration.raw_configuration, output)
    LOGGER.info("✍  %s Writing terraform output files ...%s", BOLD, ENDC)
    generator.create_templates()
    aws_provider = configuration.raw_configuration["providers"][0]["aws"]
    if aws_provider.get("forwarding_regions"):
        for region in aws_provider.get("forwarding_regions"):
            output_file = REGION_DEFAULT + "_" + region.replace("-", "_")
            generator = RegionTemplateGenerator(
                configuration.raw_configuration, output_file, region
            )
            LOGGER.info("✍  %s Writing terraform output files ...%s", BOLD, ENDC)
            generator.create_templates()



import os

from setuptools import setup

setup(
    name="reflex-cli",
    version=f"{os.environ['VERSION']}",
    packages=["reflex_cli", "reflex_cli.commands"],
    include_package_data=True,
    install_requires=[
        "click",
        "jinja2",
        "pyyaml",
        "requests",
        "pyhcl",
        "pyinquirer",
        "pygithub",
    ],
    entry_points="""
        [console_scripts]
        reflex=reflex_cli.cli:cli
    """,
)



import os
import unittest
from unittest.mock import MagicMock, patch

from reflex_cli.terraform_skeleton import TerraformSkeleton


class TerraformSkeletonTestCase(unittest.TestCase):
    @patch("reflex_cli.terraform_skeleton.RuleDiscoverer.collect_engine")
    def setUp(self, rule_discoverer_mock):
        rule_discoverer_mock.return_value = {
            "reflex-engine": {"version": "v1.0.0"}
        }
        configuration_object = {
            "rule_name": "unit-test-rule",
            "class_name": "UnitTestRule",
            "mode": "detect",
        }
        self.terraform_skeleton = TerraformSkeleton(
            output_directory="test_dir", configuration=configuration_object
        )

    @patch("reflex_cli.terraform_skeleton.write_template_file")
    def test_create_template(self, write_template_file):
        self.terraform_skeleton.create_template(
            "cwe.tf",
            {
                "rule_class_name": self.terraform_skeleton.class_name,
                "engine_version": self.terraform_skeleton.engine_version,
            },
            "terraform/cwe/cwe.tf",
        )
        write_template_file.assert_called_once()



""" Remove repeated path functions into own file. """

import logging
from pathlib import Path

LOGGER = logging.getLogger(__name__)


def ensure_output_directory_exists(output_directory):  # pragma: no cover
    """Ensure that the path to the output directory exists."""
    Path(output_directory).mkdir(parents=True, exist_ok=True)
    directory_list = [
        "/source",
        "/.github/workflows",
        "/terraform/cwe",
        "/terraform/sqs_lambda",
    ]
    for directory in directory_list:
        Path(output_directory + directory).mkdir(parents=True, exist_ok=True)


def write_template_file(output_file, rendered_template):  # pragma: no cover
    """Writes output of rendering to file"""
    LOGGER.info("Creating %s", output_file)
    with open(output_file, "w+") as file_handler:
        file_handler.write(rendered_template)



"""Entrypoint class for accumulating subcommands by project structure."""
# pylint: disable=arguments-differ
# pylint: disable=unused-argument,inconsistent-return-statements
import logging
import os
import sys
import click

CMD_FOLDER = os.path.abspath(os.path.join(os.path.dirname(__file__), "commands"))


class ReflexCli(click.MultiCommand):
    """Class that handles subcommand setup."""

    @staticmethod
    def is_command_file(filename):
        """Determines based on file name if command file."""
        if filename.endswith(".py") and filename.startswith("command_"):
            return True
        return False

    def list_commands(self, context):
        command_files = []
        for filename in os.listdir(CMD_FOLDER):
            if self.is_command_file(filename):
                command_files.append(
                    filename[8:-3]
                )  # Remove "command_" prefix and ".py" suffix
        command_files.sort()
        return command_files

    def get_command(self, context, name):
        try:

            mod = __import__(f"reflex_cli.commands.command_{name}", None, None, ["cli"])
        except ImportError:
            logging.error("%s is not a command for reflex.", name)
            sys.exit(77)
        return mod.cli
